@@ -141,6 +141,10 @@ public class ArtemisCreatePlugin extends ArtemisAbstractPlugin {
    @Parameter
    private String[] webListWithDeps;
 
+   /** Folders with libs to be copied into target */
+   @Parameter()
+   private String[] libFolders;
+
    @Parameter(defaultValue = "${localRepository}")
    private org.apache.maven.artifact.repository.ArtifactRepository localRepository;
 
@@ -200,7 +204,7 @@ public class ArtemisCreatePlugin extends ArtemisAbstractPlugin {
       }
 
       if (staticCluster != null) {
-         add(listCommands, "--staticCluster", staticCluster);
+         add(listCommands, "--static-cluster", staticCluster);
       }
 
       if (!javaOptions.isEmpty()) {
@@ -264,7 +268,7 @@ public class ArtemisCreatePlugin extends ArtemisAbstractPlugin {
          commandLineStream.println("# These are the commands used to create " + instance.getName());
          commandLineStream.println(getCommandline(listCommands));
 
-         Artemis.execute(home, null, useSystemOutput, listCommands);
+         Artemis.execute(home, null, null, useSystemOutput, listCommands);
 
          if (configuration != null) {
             String[] list = configuration.list();
@@ -301,6 +305,17 @@ public class ArtemisCreatePlugin extends ArtemisAbstractPlugin {
 
          FileUtil.makeExec(commandLine);
 
+         if (libFolders != null) {
+            for (String libFolder : libFolders) {
+               File folder = new File(libFolder);
+               for (File file : folder.listFiles()) {
+                  if (!file.isDirectory()) {
+                     copyToDir("lib", file, commandLineStream);
+                  }
+               }
+            }
+         }
+
          if (getLog().isDebugEnabled()) {
             getLog().debug("###################################################################################################");
             getLog().debug(commandLine.getName() + " created with commands to reproduce " + instance.getName());
