@@ -1,20 +1,19 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements. See the NOTICE file distributed with
+ * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * <p>
- * http://www.apache.org/licenses/LICENSE-2.0
- * <p>
+ * the License.  You may obtain a copy of the License at
+ *
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.activemq.artemis.tests.integration.amqp.connect;
 
 import javax.jms.Connection;
@@ -68,6 +67,7 @@ public class BrokerInSyncTest extends AmqpClientTestSupport {
    protected static final int AMQP_PORT_2 = 5673;
    private static final Logger logger = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
    ActiveMQServer server_2;
+   private AssertionLoggerHandler loggerHandler;
 
    @After
    public void stopServer1() throws Exception {
@@ -85,15 +85,16 @@ public class BrokerInSyncTest extends AmqpClientTestSupport {
 
    @Before
    public void startLogging() {
-      AssertionLoggerHandler.startCapture();
+      loggerHandler = new AssertionLoggerHandler();
+
    }
 
    @After
-   public void stopLogging() {
+   public void stopLogging() throws Exception {
       try {
-         Assert.assertFalse(AssertionLoggerHandler.findText("AMQ222214"));
+         Assert.assertFalse(loggerHandler.findText("AMQ222214"));
       } finally {
-         AssertionLoggerHandler.stopCapture();
+         loggerHandler.close();
       }
    }
 
@@ -468,8 +469,8 @@ public class BrokerInSyncTest extends AmqpClientTestSupport {
 
    @Test
    public void testLVQ() throws Exception {
-      AssertionLoggerHandler.startCapture();
-      runAfter(AssertionLoggerHandler::stopCapture);
+      AssertionLoggerHandler loggerHandler = new AssertionLoggerHandler();
+      runAfter(() -> loggerHandler.close());
 
       server.setIdentity("Server1");
       {
@@ -542,7 +543,7 @@ public class BrokerInSyncTest extends AmqpClientTestSupport {
       server_2.stop();
       server.stop();
 
-      Assert.assertFalse(AssertionLoggerHandler.findText("AMQ222153"));
+      Assert.assertFalse(loggerHandler.findText("AMQ222153"));
    }
 
 
@@ -938,4 +939,4 @@ public class BrokerInSyncTest extends AmqpClientTestSupport {
       }
    }
 
-}
\ No newline at end of file
+}
