@@ -1,12 +1,12 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements. See the NOTICE file distributed with
+ * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
+ * the License.  You may obtain a copy of the License at
  *
- * http://www.apache.org/licenses/LICENSE-2.0
+ *      http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -14,7 +14,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.activemq.artemis.tests.integration.client;
 
 import javax.transaction.xa.Xid;
@@ -62,6 +61,8 @@ import org.apache.activemq.artemis.core.persistence.QueueBindingInfo;
 import org.apache.activemq.artemis.core.persistence.AddressQueueStatus;
 import org.apache.activemq.artemis.core.persistence.StorageManager;
 import org.apache.activemq.artemis.core.persistence.config.PersistedAddressSetting;
+import org.apache.activemq.artemis.core.persistence.config.PersistedBridgeConfiguration;
+import org.apache.activemq.artemis.core.persistence.config.PersistedConnector;
 import org.apache.activemq.artemis.core.persistence.config.PersistedDivertConfiguration;
 import org.apache.activemq.artemis.core.persistence.config.PersistedKeyValuePair;
 import org.apache.activemq.artemis.core.persistence.config.PersistedRole;
@@ -599,9 +600,10 @@ public class SendAckFailTest extends SpawnedTestBase {
                                                        Map<Long, QueueBindingInfo> queueInfos,
                                                        Map<SimpleString, List<Pair<byte[], Long>>> duplicateIDMap,
                                                        Set<Pair<Long, Long>> pendingLargeMessages,
+                                                       Set<Long> largeMessagesInFolder,
                                                        List<PageCountPending> pendingNonTXPageCounter,
                                                        JournalLoader journalLoader) throws Exception {
-         return manager.loadMessageJournal(postOffice, pagingManager, resourceManager, queueInfos, duplicateIDMap, pendingLargeMessages, pendingNonTXPageCounter, journalLoader);
+         return manager.loadMessageJournal(postOffice, pagingManager, resourceManager, queueInfos, duplicateIDMap, pendingLargeMessages, largeMessagesInFolder, pendingNonTXPageCounter, journalLoader);
       }
 
       @Override
@@ -717,6 +719,32 @@ public class SendAckFailTest extends SpawnedTestBase {
          return null;
       }
 
+      @Override
+      public void storeBridgeConfiguration(PersistedBridgeConfiguration persistedBridgeConfiguration) throws Exception {
+      }
+
+      @Override
+      public void deleteBridgeConfiguration(String bridgeName) throws Exception {
+      }
+
+      @Override
+      public List<PersistedBridgeConfiguration> recoverBridgeConfigurations() {
+         return null;
+      }
+
+      @Override
+      public void storeConnector(PersistedConnector persistedConnector) throws Exception {
+      }
+
+      @Override
+      public void deleteConnector(String connectorName) throws Exception {
+      }
+
+      @Override
+      public List<PersistedConnector> recoverConnectors() {
+         return null;
+      }
+
       @Override
       public void storeUser(PersistedUser persistedUser) throws Exception {
          manager.storeUser(persistedUser);
@@ -881,4 +909,4 @@ public class SendAckFailTest extends SpawnedTestBase {
       }
    }
 
-}
\ No newline at end of file
+}
