@@ -19,6 +19,8 @@ package org.apache.activemq.artemis.tests.unit.core.journal.impl;
 import java.io.File;
 import java.nio.ByteBuffer;
 import java.util.List;
+import java.util.concurrent.CountDownLatch;
+import java.util.concurrent.TimeUnit;
 
 import org.apache.activemq.artemis.api.core.ActiveMQException;
 import org.apache.activemq.artemis.api.core.ActiveMQIOErrorException;
@@ -27,15 +29,20 @@ import org.apache.activemq.artemis.core.journal.EncodingSupport;
 import org.apache.activemq.artemis.core.journal.RecordInfo;
 import org.apache.activemq.artemis.core.journal.TestableJournal;
 import org.apache.activemq.artemis.core.journal.impl.JournalImpl;
-import org.apache.activemq.artemis.tests.unit.UnitTestLogger;
+import org.apache.activemq.artemis.logs.AssertionLoggerHandler;
 import org.apache.activemq.artemis.tests.unit.core.journal.impl.fakes.SimpleEncoding;
 import org.apache.activemq.artemis.utils.RandomUtil;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+import java.lang.invoke.MethodHandles;
 import org.junit.After;
 import org.junit.Assert;
 import org.junit.Test;
 
 public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
+   private static final Logger logger = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
+
    @Override
    @After
    public void tearDown() throws Exception {
@@ -111,6 +118,31 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       stopJournal();
    }
 
+   @Test
+   public void testFlushAppendsAndDeletes() throws Exception {
+      setup(10, 10 * 1024, true);
+      createJournal();
+      startJournal();
+      load();
+      byte[] record = new byte[1000];
+      for (int i = 0; i < record.length; i++) {
+         record[i] = (byte) 'a';
+      }
+      // Appending records after restart should be valid (not throwing any
+      // exceptions)
+      for (int i = 0; i < 10_000; i++) {
+         journal.appendAddRecord(i, (byte) 1, new SimpleEncoding(2, (byte) 'a'), false);
+         journal.appendDeleteRecord(i, false);
+      }
+      stopJournal();
+
+      List<String> files = fileFactory.listFiles(fileExtension);
+
+      // I am allowing one extra as a possible race with pushOpenFiles. I have not seen it happening on my test
+      // but it wouldn't be a problem if it happened
+      Assert.assertTrue("Supposed to have up to 10 files", files.size() <= 11);
+   }
+
    @Test
    public void testParams() throws Exception {
       try {
@@ -428,6 +460,33 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       stopJournal();
    }
 
+
+   @Test
+   public void testReduceFreeFilesWithCleanup() throws Exception {
+      setup(10, 10 * 1024, true);
+      createJournal();
+      startJournal();
+      load();
+
+      List<String> files1 = fileFactory.listFiles(fileExtension);
+
+      Assert.assertEquals(10, files1.size());
+
+      stopJournal();
+
+      setup(5, 10 * 1024, true);
+      createJournal();
+      journal.setRemoveExtraFilesOnLoad(true);
+      startJournal();
+      load();
+
+      List<String> files2 = fileFactory.listFiles(fileExtension);
+
+      Assert.assertEquals(5, files2.size());
+
+      stopJournal();
+   }
+
    private int calculateRecordsPerFile(final int fileSize, final int alignment, int recordSize) {
       recordSize = calculateRecordSize(recordSize, alignment);
       int headerSize = calculateRecordSize(JournalImpl.SIZE_HEADER, alignment);
@@ -437,7 +496,10 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
    /**
     * Use: calculateNumberOfFiles (fileSize, numberOfRecords, recordSize,  numberOfRecords2, recordSize2, , ...., numberOfRecordsN, recordSizeN);
     */
-   private int calculateNumberOfFiles(TestableJournal journal, final int fileSize, final int alignment, final int... record) throws Exception {
+   private int calculateNumberOfFiles(TestableJournal journal,
+                                      final int fileSize,
+                                      final int alignment,
+                                      final int... record) throws Exception {
       if (journal != null) {
          journal.flush();
       }
@@ -496,6 +558,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       Assert.assertEquals(0, journal.getFreeFilesCount());
       Assert.assertEquals(91, journal.getIDMapSize());
 
+      journal.processBackup();
       List<String> files2 = fileFactory.listFiles(fileExtension);
 
       // The Journal will aways have a file ready to be opened
@@ -540,6 +603,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       Assert.assertEquals(0, journal.getFreeFilesCount());
       Assert.assertEquals(200, journal.getIDMapSize());
 
+      journal.processBackup();
       List<String> files4 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(numberOfFiles + 2, files4.size());
@@ -597,10 +661,6 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       files1 = fileFactory.listFiles(fileExtension);
       Assert.assertTrue(files1.size() > 200);
 
-      Assert.assertEquals(numberOfFiles, files1.size());
-
-      System.out.println("we have " + files1.size() + " files now");
-
       stopJournal();
    }
 
@@ -675,7 +735,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       int addRecordsPerFile = calculateRecordsPerFile(10 * 1024, journal.getAlignment(), JournalImpl.SIZE_ADD_RECORD + 1 + recordLength);
 
-      System.out.println(JournalImpl.SIZE_ADD_RECORD + 1 + recordLength);
+      logger.debug("{}", (JournalImpl.SIZE_ADD_RECORD + 1 + recordLength));
 
       // Fills exactly 10 files
       int initialNumberOfAddRecords = addRecordsPerFile * 10;
@@ -689,6 +749,8 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       Assert.assertEquals(0, journal.getFreeFilesCount());
       Assert.assertEquals(initialNumberOfAddRecords, journal.getIDMapSize());
 
+      journal.processBackup();
+
       List<String> files4 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(11, files4.size());
@@ -770,6 +832,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       update(1);
       delete(1);
 
+      journal.processBackup();
       List<String> files1 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(4, files1.size());
@@ -809,6 +872,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       update(1);
       add(2);
 
+      journal.processBackup();
       List<String> files1 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(4, files1.size());
@@ -863,6 +927,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
          addTx(1, i);
       }
 
+      journal.processBackup();
       Assert.assertEquals(calculateNumberOfFiles(journal, fileSize, journal.getAlignment(), 100, recordLength), journal.getDataFilesCount());
       Assert.assertEquals(0, journal.getFreeFilesCount());
       Assert.assertEquals(0, journal.getIDMapSize());
@@ -884,6 +949,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       Assert.assertEquals(0, journal.getFreeFilesCount());
       Assert.assertEquals(0, journal.getIDMapSize());
 
+      journal.processBackup();
       List<String> files3 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(calculateNumberOfFiles(journal, fileSize, journal.getAlignment(), 100, recordLength) + 2, files3.size());
@@ -903,6 +969,8 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       Assert.assertEquals(0, journal.getFreeFilesCount());
       Assert.assertEquals(0, journal.getIDMapSize());
 
+
+      journal.processBackup();
       List<String> files4 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(calculateNumberOfFiles(journal, fileSize, journal.getAlignment(), 200, recordLength) + 2, files4.size());
@@ -916,10 +984,10 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       // Make sure nothing reclaimed
 
-      Assert.assertEquals(calculateNumberOfFiles(journal, fileSize, journal.getAlignment(), 200, recordLength), journal.getDataFilesCount());
       Assert.assertEquals(0, journal.getFreeFilesCount());
       Assert.assertEquals(0, journal.getIDMapSize());
 
+      journal.processBackup();
       List<String> files5 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(24, files5.size());
@@ -935,14 +1003,12 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
          deleteTx(1, i);
       }
 
-      Assert.assertEquals(calculateNumberOfFiles(journal, fileSize, journal.getAlignment(), 200, recordLength, 200, JournalImpl.SIZE_DELETE_RECORD_TX), journal.getDataFilesCount());
-
       Assert.assertEquals(0, journal.getFreeFilesCount());
       Assert.assertEquals(0, journal.getIDMapSize());
 
+      journal.processBackup();
       List<String> files7 = fileFactory.listFiles(fileExtension);
 
-      Assert.assertEquals(calculateNumberOfFiles(journal, fileSize, journal.getAlignment(), 200, recordLength, 200, JournalImpl.SIZE_DELETE_RECORD_TX) + 2, files7.size());
       Assert.assertEquals(1, journal.getOpenedFilesCount());
 
       for (String file : files1) {
@@ -951,13 +1017,12 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       checkAndReclaimFiles();
 
-      Assert.assertEquals(calculateNumberOfFiles(journal, fileSize, journal.getAlignment(), 200, recordLength, 200, JournalImpl.SIZE_DELETE_RECORD_TX), journal.getDataFilesCount());
       Assert.assertEquals(0, journal.getFreeFilesCount());
       Assert.assertEquals(0, journal.getIDMapSize());
 
+      journal.processBackup();
       List<String> files8 = fileFactory.listFiles(fileExtension);
 
-      Assert.assertEquals(calculateNumberOfFiles(journal, fileSize, journal.getAlignment(), 200, recordLength, 200, JournalImpl.SIZE_DELETE_RECORD_TX) + 2, files8.size());
       Assert.assertEquals(1, journal.getOpenedFilesCount());
 
       for (String file : files1) {
@@ -978,13 +1043,12 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
          add(i);
       }
 
-      Assert.assertEquals(calculateNumberOfFiles(journal, fileSize, journal.getAlignment(), 200, recordLength, 200, JournalImpl.SIZE_DELETE_RECORD_TX, 1, JournalImpl.SIZE_COMMIT_RECORD, 10, JournalImpl.SIZE_ADD_RECORD + recordLength), journal.getDataFilesCount());
       Assert.assertEquals(0, journal.getFreeFilesCount());
       Assert.assertEquals(10, journal.getIDMapSize());
 
+      journal.processBackup();
       List<String> files9 = fileFactory.listFiles(fileExtension);
 
-      Assert.assertEquals(calculateNumberOfFiles(journal, fileSize, journal.getAlignment(), 200, recordLength, 200, JournalImpl.SIZE_DELETE_RECORD_TX, 1, JournalImpl.SIZE_COMMIT_RECORD, 10, JournalImpl.SIZE_ADD_RECORD + recordLength) + 2, files9.size());
       Assert.assertEquals(1, journal.getOpenedFilesCount());
 
       for (String file : files1) {
@@ -995,16 +1059,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       // Most Should now be reclaimed - leaving 10 left in total
 
-      Assert.assertEquals(journal.getAlignment() == 1 ? 1 : 2, journal.getDataFilesCount());
-      Assert.assertEquals(journal.getAlignment() == 1 ? 7 : 6, journal.getFreeFilesCount());
       Assert.assertEquals(10, journal.getIDMapSize());
-
-      List<String> files10 = fileFactory.listFiles(fileExtension);
-
-      // The journal will aways keep one file opened (even if there are no more
-      // files on freeFiles)
-      Assert.assertEquals(10, files10.size());
-      Assert.assertEquals(1, journal.getOpenedFilesCount());
    }
 
    @Test
@@ -1013,6 +1068,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       createJournal();
       startJournal();
       load();
+      journal.processBackup();
       List<String> files1 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(2, files1.size());
@@ -1028,8 +1084,11 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       journal.debugWait();
 
-      System.out.println("journal tmp :" + journal.debug());
+      if (logger.isDebugEnabled()) {
+         logger.debug("journal tmp : {}", journal.debug());
+      }
 
+      journal.processBackup();
       List<String> files2 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(3, files2.size());
@@ -1045,15 +1104,18 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       journal.debugWait();
 
-      System.out.println("journal tmp2 :" + journal.debug());
+      if (logger.isDebugEnabled()) {
+         logger.debug("journal tmp2 : {}", journal.debug());
+      }
 
+      journal.processBackup();
       List<String> files3 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(4, files3.size());
       Assert.assertEquals(1, journal.getOpenedFilesCount());
 
-      UnitTestLogger.LOGGER.debug("data files count " + journal.getDataFilesCount());
-      UnitTestLogger.LOGGER.debug("free files count " + journal.getFreeFilesCount());
+      logger.debug("data files count {}", journal.getDataFilesCount());
+      logger.debug("free files count {}", journal.getFreeFilesCount());
 
       Assert.assertEquals(2, journal.getDataFilesCount());
       Assert.assertEquals(0, journal.getFreeFilesCount());
@@ -1061,6 +1123,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       commit(1); // in file 3
 
+      journal.processBackup();
       List<String> files4 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(5, files4.size());
@@ -1074,6 +1137,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       addWithSize(recordLength - JournalImpl.SIZE_ADD_RECORD - 1, 3); // in file 4
 
+      journal.processBackup();
       List<String> files5 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(6, files5.size());
@@ -1085,6 +1149,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       checkAndReclaimFiles();
 
+      journal.processBackup();
       List<String> files6 = fileFactory.listFiles(fileExtension);
 
       // Three should get deleted (files 0, 1, 3)
@@ -1100,8 +1165,9 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       journal.checkReclaimStatus();
 
-      System.out.println("journal:" + journal.debug());
-
+      if (logger.isDebugEnabled()) {
+         logger.debug("journal: {}", journal.debug());
+      }
       stopJournal(false);
       createJournal();
       startJournal();
@@ -1122,6 +1188,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       startJournal();
       load();
 
+      journal.processBackup();
       List<String> files1 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(10, files1.size());
@@ -1160,6 +1227,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       startJournal();
       load();
 
+      journal.processBackup();
       List<String> files1 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(10, files1.size());
@@ -1191,6 +1259,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       startJournal();
       load();
 
+      journal.processBackup();
       List<String> files1 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(10, files1.size());
@@ -1229,6 +1298,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       startJournal();
       load();
 
+      journal.processBackup();
       List<String> files1 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(10, files1.size());
@@ -1267,6 +1337,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       startJournal();
       load();
 
+      journal.processBackup();
       List<String> files1 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(10, files1.size());
@@ -1298,6 +1369,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       startJournal();
       load();
 
+      journal.processBackup();
       List<String> files1 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(10, files1.size());
@@ -1336,6 +1408,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       startJournal();
       load();
 
+      journal.processBackup();
       List<String> files1 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(10, files1.size());
@@ -1368,6 +1441,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       startJournal();
       load();
 
+      journal.processBackup();
       List<String> files1 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(2, files1.size());
@@ -1379,6 +1453,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       addTx(1, 1);
 
+      journal.processBackup();
       List<String> files2 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(2, files2.size());
@@ -1392,6 +1467,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       commit(1);
 
+      journal.processBackup();
       List<String> files3 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(3, files3.size());
@@ -1405,6 +1481,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       // Move on to another file
 
+      journal.processBackup();
       List<String> files4 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(4, files4.size());
@@ -1418,6 +1495,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       // Nothing should be reclaimed
 
+      journal.processBackup();
       List<String> files5 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(4, files5.size());
@@ -1434,12 +1512,12 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
    @Test
    public void testCommitRecordsInFileNoReclaim() throws Exception {
-      setup(2, calculateRecordSize(JournalImpl.SIZE_HEADER, getAlignment()) + calculateRecordSize(recordLength, getAlignment()) +
-         512, true);
+      setup(2, calculateRecordSize(JournalImpl.SIZE_HEADER, getAlignment()) + calculateRecordSize(recordLength, getAlignment()) + 512, true);
       createJournal();
       startJournal();
       load();
 
+      journal.processBackup();
       List<String> files1 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(2, files1.size());
@@ -1455,6 +1533,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       addWithSize(recordLength - JournalImpl.SIZE_ADD_RECORD - 1, 2); // in file 1
 
+      journal.processBackup();
       List<String> files2 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(3, files2.size());
@@ -1466,6 +1545,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       commit(1); // in file 1
 
+      journal.processBackup();
       List<String> files3 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(calculateNumberOfFiles(journal, fileSize, journal.getAlignment(), 2, recordLength, 1, JournalImpl.SIZE_COMMIT_RECORD + 1) + 2, files3.size());
@@ -1477,6 +1557,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       delete(2); // in file 1
 
+      journal.processBackup();
       List<String> files4 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(calculateNumberOfFiles(journal, fileSize, journal.getAlignment(), 2, recordLength, 1, JournalImpl.SIZE_COMMIT_RECORD + 1, 1, JournalImpl.SIZE_DELETE_RECORD + 1) + 2, files4.size());
@@ -1490,6 +1571,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       addWithSize(recordLength - JournalImpl.SIZE_ADD_RECORD, 3); // in file 2
 
+      journal.processBackup();
       List<String> files5 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(0, journal.getFreeFilesCount());
@@ -1498,6 +1580,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       checkAndReclaimFiles();
 
+      journal.processBackup();
       List<String> files6 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(0, journal.getFreeFilesCount());
@@ -1518,12 +1601,12 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
    @Test
    public void testRollbackRecordsInFileNoReclaim() throws Exception {
-      setup(2, calculateRecordSize(JournalImpl.SIZE_HEADER, getAlignment()) + calculateRecordSize(recordLength, getAlignment()) +
-         512, true);
+      setup(2, calculateRecordSize(JournalImpl.SIZE_HEADER, getAlignment()) + calculateRecordSize(recordLength, getAlignment()) + 512, true);
       createJournal();
       startJournal();
       load();
 
+      journal.processBackup();
       List<String> files1 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(2, files1.size());
@@ -1539,6 +1622,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       addWithSize(recordLength - JournalImpl.SIZE_ADD_RECORD - 1, 2); // in file 1
 
+      journal.processBackup();
       List<String> files2 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(3, files2.size());
@@ -1559,6 +1643,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       delete(2); // in file 1
 
+      journal.processBackup();
       List<String> files4 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(calculateNumberOfFiles(journal, fileSize, journal.getAlignment(), 2, recordLength, 1, JournalImpl.SIZE_ROLLBACK_RECORD + 1, 1, JournalImpl.SIZE_DELETE_RECORD + 1) + 2, files4.size());
@@ -1580,6 +1665,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       checkAndReclaimFiles();
 
+      journal.processBackup();
       List<String> files6 = fileFactory.listFiles(fileExtension);
 
       // files 0 and 1 should be deleted
@@ -1598,6 +1684,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       startJournal();
       loadAndCheck();
 
+      journal.processBackup();
       List<String> files7 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(journal.getAlignment() == 1 ? 2 : 3, files7.size());
@@ -1610,8 +1697,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
    @Test
    public void testEmptyPrepare() throws Exception {
-      setup(2, calculateRecordSize(JournalImpl.SIZE_HEADER, getAlignment()) + calculateRecordSize(recordLength, getAlignment()) +
-         512, true);
+      setup(2, calculateRecordSize(JournalImpl.SIZE_HEADER, getAlignment()) + calculateRecordSize(recordLength, getAlignment()) + 512, true);
       createJournal();
       startJournal();
       load();
@@ -1645,12 +1731,12 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
    @Test
    public void testPrepareNoReclaim() throws Exception {
-      setup(2, calculateRecordSize(JournalImpl.SIZE_HEADER, getAlignment()) + calculateRecordSize(recordLength, getAlignment()) +
-         512, true);
+      setup(2, calculateRecordSize(JournalImpl.SIZE_HEADER, getAlignment()) + calculateRecordSize(recordLength, getAlignment()) + 512, true);
       createJournal();
       startJournal();
       load();
 
+      journal.processBackup();
       List<String> files1 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(2, files1.size());
@@ -1666,6 +1752,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       addWithSize(1024 - JournalImpl.SIZE_ADD_RECORD, 2); // in file 1
 
+      journal.processBackup();
       List<String> files2 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(3, files2.size());
@@ -1700,6 +1787,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       checkAndReclaimFiles();
 
+      journal.processBackup();
       List<String> files6 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(4, files6.size());
@@ -1711,6 +1799,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       addWithSize(recordLength - JournalImpl.SIZE_ADD_RECORD - 1, 4); // in file 3
 
+      journal.processBackup();
       List<String> files7 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(5, files7.size());
@@ -1722,6 +1811,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       commit(1); // in file 4
 
+      journal.processBackup();
       List<String> files8 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(5, files8.size());
@@ -1747,6 +1837,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       startJournal();
       load();
 
+      journal.processBackup();
       List<String> files1 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(2, files1.size());
@@ -1758,6 +1849,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       addTx(1, 1); // in file 0
 
+      journal.processBackup();
       files1 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(2, files1.size());
@@ -1775,6 +1867,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       addWithSize(recordLength - JournalImpl.SIZE_ADD_RECORD, 2); // in file 1
 
+      journal.processBackup();
       List<String> files2 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(3, files2.size());
@@ -1786,6 +1879,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       EncodingSupport xid = new SimpleEncoding(10, (byte) 0);
       prepare(1, xid); // in file 1
 
+      journal.processBackup();
       List<String> files3 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(3, files3.size());
@@ -1797,6 +1891,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       delete(2); // in file 1
 
+      journal.processBackup();
       List<String> files4 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(3, files4.size());
@@ -1815,6 +1910,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       checkAndReclaimFiles();
 
+      journal.processBackup();
       List<String> files5 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(4, files5.size());
@@ -1826,6 +1922,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       checkAndReclaimFiles();
 
+      journal.processBackup();
       List<String> files6 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(4, files6.size());
@@ -1839,6 +1936,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       addWithSize(1024 - JournalImpl.SIZE_ADD_RECORD, 4); // in file 3
 
+      journal.processBackup();
       List<String> files7 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(5, files7.size());
@@ -1850,6 +1948,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       commit(1); // in file 3
 
+      journal.processBackup();
       List<String> files8 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(5, files8.size());
@@ -1861,6 +1960,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       delete(1); // in file 3
 
+      journal.processBackup();
       List<String> files9 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(5, files9.size());
@@ -1872,6 +1972,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       checkAndReclaimFiles();
 
+      journal.processBackup();
       List<String> files10 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(journal.getAlignment() == 1 ? 5 : 5, files10.size());
@@ -1884,6 +1985,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       addWithSize(1024 - JournalImpl.SIZE_ADD_RECORD, 5); // in file 4
 
+      journal.processBackup();
       List<String> files11 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(6, files11.size());
@@ -1895,6 +1997,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       checkAndReclaimFiles();
 
+      journal.processBackup();
       List<String> files12 = fileFactory.listFiles(fileExtension);
 
       // File 0, and File 1 should be deleted
@@ -1922,7 +2025,9 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       addWithSize(1024 - JournalImpl.SIZE_ADD_RECORD, 6);
 
-      UnitTestLogger.LOGGER.debug("Debug journal on testPrepareReclaim ->\n" + debugJournal());
+      if (logger.isDebugEnabled()) {
+         logger.debug("Debug journal on testPrepareReclaim ->\n{}", debugJournal());
+      }
 
       Assert.assertEquals(1, journal.getOpenedFilesCount());
       Assert.assertEquals(0, journal.getFreeFilesCount());
@@ -1932,6 +2037,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
       // file 3 should now be deleted
 
+      journal.processBackup();
       List<String> files15 = fileFactory.listFiles(fileExtension);
 
       Assert.assertEquals(4, files15.size());
@@ -2019,6 +2125,57 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       loadAndCheck();
    }
 
+   @Test
+   public void testDoubleDelete() throws Exception {
+
+      try (AssertionLoggerHandler loggerHandler = new AssertionLoggerHandler(true)) {
+         setup(10, 10 * 1024, true);
+         createJournal();
+         startJournal();
+         load();
+
+         byte[] record = generateRecord(100);
+
+         add(1);
+
+         // I'm not adding that to the test assertion, as it will be deleted anyway.
+         // the test assertion doesn't support multi-thread, so I'm calling the journal directly here
+         journal.appendAddRecord(2, (byte) 0, record, sync);
+
+         Thread[] threads = new Thread[100];
+         CountDownLatch alignLatch = new CountDownLatch(threads.length);
+         CountDownLatch startFlag = new CountDownLatch(1);
+         for (int i = 0; i < threads.length; i++) {
+            threads[i] = new Thread(() -> {
+               alignLatch.countDown();
+               try {
+                  startFlag.await(5, TimeUnit.SECONDS);
+                  journal.appendDeleteRecord(2, false);
+               } catch (java.lang.IllegalStateException expected) {
+               } catch (Exception e) {
+                  // e.printStackTrace();
+               }
+            });
+            threads[i].start();
+         }
+
+         Assert.assertTrue(alignLatch.await(5, TimeUnit.SECONDS));
+         startFlag.countDown();
+
+         for (Thread t : threads) {
+            t.join(TimeUnit.SECONDS.toMillis(10));
+            Assert.assertFalse(t.isAlive());
+         }
+         journal.flush();
+
+         Assert.assertFalse(loggerHandler.findTrace("NullPointerException"));
+         stopJournal();
+         createJournal();
+         startJournal();
+         loadAndCheck();
+      }
+   }
+
    @Test
    public void testMultipleAddUpdateAll() throws Exception {
       setup(10, 10 * 1024, true);
@@ -2181,7 +2338,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
          journal.appendAddRecord(i, (byte) 0, record, false);
 
-         records.add(new RecordInfo(i, (byte) 0, record, false, (short) 0));
+         records.add(new RecordInfo(i, (byte) 0, record, false, false, (short) 0));
       }
 
       for (int i = 0; i < 100; i++) {
@@ -2189,7 +2346,7 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
 
          journal.appendUpdateRecord(i, (byte) 0, record, false);
 
-         records.add(new RecordInfo(i, (byte) 0, record, true, (short) 0));
+         records.add(new RecordInfo(i, (byte) 0, record, true, false, (short) 0));
       }
 
       for (int i = 0; i < 100; i++) {
@@ -2653,6 +2810,22 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       loadAndCheck();
    }
 
+   @Test
+   public void testTryIsolation2() throws Exception {
+      setup(10, 10 * 1024, true);
+      createJournal();
+      startJournal();
+      load();
+      addTx(1, 1, 2, 3);
+
+      Assert.assertFalse(tryUpdate(1));
+
+      stopJournal();
+      createJournal();
+      startJournal();
+      loadAndCheck();
+   }
+
    @Test
    public void testIsolation3() throws Exception {
       setup(10, 10 * 1024, true);
@@ -2673,6 +2846,22 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       loadAndCheck();
    }
 
+   @Test
+   public void testTryDelete() throws Exception {
+      setup(10, 10 * 1024, true);
+      createJournal();
+      startJournal();
+      load();
+      addTx(1, 1, 2, 3);
+
+      Assert.assertFalse(tryDelete(1));
+
+      stopJournal();
+      createJournal();
+      startJournal();
+      loadAndCheck();
+   }
+
    // XA tests
    // ========
 
@@ -2875,7 +3064,9 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
       addTx(4, 31);
       commit(3);
 
-      UnitTestLogger.LOGGER.debug("Debug on Journal before stopJournal - \n" + debugJournal());
+      if (logger.isDebugEnabled()) {
+         logger.debug("Debug on Journal before stopJournal - \n{}", debugJournal());
+      }
 
       stopJournal();
       createJournal();
@@ -2912,18 +3103,22 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
          update(i);
       }
 
-      System.out.println("Before stop ****************************");
-      System.out.println(journal.debug());
-      System.out.println("*****************************************");
+      /**
+       * Enable System.outs again if test fails and needs to be debugged
+       */
+
+      //      logger.debug("Before stop ****************************");
+      //      logger.debug(journal.debug());
+      //      logger.debug("*****************************************");
 
       stopJournal();
       createJournal();
       startJournal();
       loadAndCheck();
 
-      System.out.println("After start ****************************");
-      System.out.println(journal.debug());
-      System.out.println("*****************************************");
+      //      logger.debug("After start ****************************");
+      //      logger.debug(journal.debug());
+      //      logger.debug("*****************************************");
 
       journal.forceMoveNextFile();
 
@@ -2931,49 +3126,50 @@ public abstract class JournalImplTestUnit extends JournalImplTestBase {
          delete(i);
       }
 
-      System.out.println("After delete ****************************");
-      System.out.println(journal.debug());
-      System.out.println("*****************************************");
+      //      logger.debug("After delete ****************************");
+      //      logger.debug(journal.debug());
+      //      logger.debug("*****************************************");
 
       for (int i = 100; i < 200; i++) {
          updateTx(transactionID, i);
       }
 
-      System.out.println("After updatetx ****************************");
-      System.out.println(journal.debug());
-      System.out.println("*****************************************");
+      //      logger.debug("After updatetx ****************************");
+      //      logger.debug(journal.debug());
+      //      logger.debug("*****************************************");
 
       journal.forceMoveNextFile();
 
       commit(transactionID++);
 
-      System.out.println("After commit ****************************");
-      System.out.println(journal.debug());
-      System.out.println("*****************************************");
+      //      logger.debug("After commit ****************************");
+      //      logger.debug(journal.debug());
+      //      logger.debug("*****************************************");
 
       for (int i = 100; i < 200; i++) {
          updateTx(transactionID, i);
          deleteTx(transactionID, i);
       }
 
-      System.out.println("After delete ****************************");
-      System.out.println(journal.debug());
-      System.out.println("*****************************************");
+      //      logger.debug("After delete ****************************");
+      //      logger.debug(journal.debug());
+      //      logger.debug("*****************************************");
 
       commit(transactionID++);
 
-      System.out.println("Before reclaim/after commit ****************************");
-      System.out.println(journal.debug());
-      System.out.println("*****************************************");
+      //      logger.debug("Before reclaim/after commit ****************************");
+      //      logger.debug(journal.debug());
+      //      logger.debug("*****************************************");
 
+      journal.processBackup();
       stopJournal();
       createJournal();
       startJournal();
       loadAndCheck();
 
-      System.out.println("After reclaim ****************************");
-      System.out.println(journal.debug());
-      System.out.println("*****************************************");
+      //      logger.debug("After reclaim ****************************");
+      //      logger.debug(journal.debug());
+      //      logger.debug("*****************************************");
 
       journal.forceMoveNextFile();
       checkAndReclaimFiles();
