@@ -46,6 +46,7 @@ import org.junit.Test;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import java.lang.invoke.MethodHandles;
+import java.util.function.BiConsumer;
 
 /**
  * This test is replicating the behaviour from https://issues.jboss.org/browse/HORNETQ-988.
@@ -433,7 +434,7 @@ public class WildcardAddressManagerUnitTest extends ActiveMQTestBase {
    static class BindingsFake implements Bindings {
 
       SimpleString name;
-      ConcurrentHashMap<SimpleString, Binding> bindings = new ConcurrentHashMap<>();
+      ConcurrentHashMap<String, Binding> bindings = new ConcurrentHashMap<>();
 
       BindingsFake(SimpleString address) {
          this.name = address;
@@ -446,12 +447,12 @@ public class WildcardAddressManagerUnitTest extends ActiveMQTestBase {
 
       @Override
       public void addBinding(Binding binding) {
-         bindings.put(binding.getUniqueName(), binding);
+         bindings.put(String.valueOf(binding.getUniqueName()), binding);
       }
 
       @Override
       public Binding removeBindingByUniqueName(SimpleString uniqueName) {
-         return bindings.remove(uniqueName);
+         return bindings.remove(String.valueOf(uniqueName));
       }
 
       @Override
@@ -464,6 +465,21 @@ public class WildcardAddressManagerUnitTest extends ActiveMQTestBase {
 
       }
 
+      @Override
+      public Binding getBinding(String name) {
+         return bindings.get(name);
+      }
+
+      @Override
+      public void forEach(BiConsumer<String, Binding> bindingConsumer) {
+         bindings.forEach(bindingConsumer);
+      }
+
+      @Override
+      public int size() {
+         return bindings.size();
+      }
+
       @Override
       public MessageLoadBalancingType getMessageLoadBalancingType() {
          return null;
@@ -478,10 +494,10 @@ public class WildcardAddressManagerUnitTest extends ActiveMQTestBase {
       }
 
       @Override
-      public boolean redistribute(Message message,
+      public Message redistribute(Message message,
                                   Queue originatingQueue,
                                   RoutingContext context) throws Exception {
-         return false;
+         return null;
       }
 
       @Override
